package com.zhn.decode.config;

import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.core.*;
import org.springframework.kafka.support.serializer.JsonDeserializer;
import org.springframework.kafka.support.serializer.JsonSerializer;

import java.util.HashMap;
import java.util.Map;

@Configuration
@EnableKafka
public class KafkaConfig {
	
	public static final String KAFKA_ADMIN_CLIENT_ID = "mqtt_to_kafka";
	@Value ("${kafka.bootstrap-servers}")
	private String bootstrapServers;
	@Value ("${kafka.consumer.group-id}")
	private String groupId;
	
	/**
	 * KafkaTemplate 配置
	 *
	 * @return KafkaTemplate 对象
	 */
	@Bean
	public KafkaTemplate<String, Object> kafkaTemplate () {
		// 返回新的 KafkaTemplate 对象
		ProducerFactory<String, Object> prop = producerFactory ();
		return new KafkaTemplate<> (prop);
	}
	
	/**
	 * 生产者工厂配置
	 *
	 * @return ProducerFactory 对象
	 */
	@Bean
	public ProducerFactory<String, Object> producerFactory () {
		// 创建 Kafka 生产者配置
		Map<String, Object> configs = new HashMap<> ();
		configs.put (ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
		configs.put (ProducerConfig.CLIENT_ID_CONFIG, KAFKA_ADMIN_CLIENT_ID);
		configs.put (ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
		configs.put (ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
		// 增加 max.poll.interval.ms 配置
		configs.put (ConsumerConfig.MAX_POLL_INTERVAL_MS_CONFIG, 300000);  // 设置为 5 分钟（单位为毫秒）
		
		// 减小 max.poll.records 配置
		configs.put (ConsumerConfig.MAX_POLL_RECORDS_CONFIG, 100);  // 设置每次 poll 返回的最大记录数为 100 条
		// 返回新的 DefaultKafkaProducerFactory 对象
		return new DefaultKafkaProducerFactory<> (configs);
	}
	
	@Bean
	public ConcurrentKafkaListenerContainerFactory<String, Object> kafkaListenerContainerFactory () {
		ConcurrentKafkaListenerContainerFactory<String, Object> factory = new ConcurrentKafkaListenerContainerFactory<> ();
		factory.setConsumerFactory (consumerFactory ());
		return factory;
	}
	
	/**
	 * 消费者工厂配置
	 *
	 * @return ConsumerFactory 对象
	 */
	@Bean
	public ConsumerFactory<String, Object> consumerFactory () {
		// 创建 Kafka 消费者配置
		Map<String, Object> props = new HashMap<> ();
		props.put (ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
		props.put (ConsumerConfig.GROUP_ID_CONFIG, groupId);
		props.put (ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
		props.put (ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, JsonDeserializer.class);
		props.put (JsonDeserializer.TRUSTED_PACKAGES, "*");
		// 返回新的 DefaultKafkaConsumerFactory 对象
		return new DefaultKafkaConsumerFactory<> (props);
	}
	
}
